@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2013 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2017 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -39,6 +39,7 @@ from Mailman.ListAdmin import readMessage
 from Mailman.Cgi import Auth
 from Mailman.htmlformat import *
 from Mailman.Logging.Syslog import syslog
+from Mailman.CSRFcheck import csrf_check
 
 EMPTYSTRING = ''
 NL = '\n'
@@ -58,6 +59,9 @@ if mm_cfg.DISPLAY_HELD_SUMMARY_SORT_BUTTONS in (SSENDERTIME, STIME):
 else:
     ssort = SSENDER
 
+AUTH_CONTEXTS = (mm_cfg.AuthListAdmin, mm_cfg.AuthSiteAdmin,
+                 mm_cfg.AuthListModerator)
+
 
 
 def helds_by_skey(mlist, ssort=SSENDER):
@@ -91,8 +95,9 @@ def hacky_radio_buttons(btnname, labels, values, defaults, spacing=3):
     space = '&nbsp;' * spacing
     btns = Table(cellspacing='5', cellpadding='0')
     btns.AddRow([space + text + space for text in labels])
-    btns.AddRow([Center(RadioButton(btnname, value, default))
-                 for value, default in zip(values, defaults)])
+    btns.AddRow([Center(RadioButton(btnname, value, default).Format()
+                     + '<div class=hidden>' + label + '</div>')
+                 for label, value, default in zip(labels, values, defaults)])
     return btns
 
 
@@ -114,7 +119,7 @@ def main():
         # Send this with a 404 status.
         print 'Status: 404 Not Found'
         handle_no_list(_('No such list <em>%(safelistname)s</em>'))
-        syslog('error', 'No such list "%s": %s\n', listname, e)
+        syslog('error', 'admindb: No such list "%s": %s\n', listname, e)
         return
 
     # Now that we know which list to use, set the system's language to it.
@@ -122,11 +127,35 @@ def main():
 
     # Make sure the user is authorized to see this page.
     cgidata = cgi.FieldStorage(keep_blank_values=1)
+    try:
+        cgidata.getfirst('adminpw', '')
+    except TypeError:
+        # Someone crafted a POST with a bad Content-Type:.
+        doc = Document()
+        doc.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
+        doc.AddItem(Header(2, _("Error")))
+        doc.AddItem(Bold(_('Invalid options to CGI script.')))
+        # Send this with a 400 status.
+        print 'Status: 400 Bad Request'
+        print doc.Format()
+        return
+
+    # CSRF check
+    safe_params = ['adminpw', 'admlogin', 'msgid', 'sender', 'details']
+    params = cgidata.keys()
+    if set(params) - set(safe_params):
+        csrf_checked = csrf_check(mlist, cgidata.getfirst('csrf_token'))
+    else:
+        csrf_checked = True
+    # if password is present, void cookie to force password authentication.
+    if cgidata.getfirst('adminpw'):
+        os.environ['HTTP_COOKIE'] = ''
+        csrf_checked = True
 
     if not mlist.WebAuthenticate((mm_cfg.AuthListAdmin,
                                   mm_cfg.AuthListModerator,
                                   mm_cfg.AuthSiteAdmin),
-                                 cgidata.getvalue('adminpw', '')):
+                                 cgidata.getfirst('adminpw', '')):
         if cgidata.has_key('adminpw'):
             # This is a re-authorization attempt
             msg = Bold(FontSize('+1', _('Authorization failed.'))).Format()
@@ -200,7 +229,11 @@ def main():
         elif not details:
             # This is a form submission
             doc.SetTitle(_('%(realname)s Administrative Database Results'))
-            process_form(mlist, doc, cgidata)
+            if csrf_checked:
+                process_form(mlist, doc, cgidata)
+            else:
+                doc.addError(
+                    _('The form lifetime has expired. (request forgery check)'))
         # Now print the results and we're done.  Short circuit for when there
         # are no pending requests, but be sure to save the results!
         admindburl = mlist.GetScriptURL('admindb', absolute=1)
@@ -213,15 +246,16 @@ def main():
             doc.AddItem(Link(admindburl,
                              _('Click here to reload this page.')))
             # Put 'Logout' link before the footer
+            doc.AddItem('\n<div align="right"><font size="+2">')
             doc.AddItem(Link('%s/logout' % admindburl,
-                '<div align="right"><font size="+2"><b>%s</b></font></div>' %
-                _('Logout')))
+                '<b>%s</b>' % _('Logout')))
+            doc.AddItem('</font></div>\n')
             doc.AddItem(mlist.GetMailmanFooter())
             print doc.Format()
             mlist.Save()
             return
 
-        form = Form(admindburl)
+        form = Form(admindburl, mlist=mlist, contexts=AUTH_CONTEXTS)
         # Add the instructions template
         if details == 'instructions':
             doc.AddItem(Header(
@@ -236,9 +270,11 @@ def main():
         nomessages = not mlist.GetHeldMessageIds()
         if not (details or sender or msgid or nomessages):
             form.AddItem(Center(
+                '<label>' +
                 CheckBox('discardalldefersp', 0).Format() +
                 '&nbsp;' +
-                _('Discard all messages marked <em>Defer</em>')
+                _('Discard all messages marked <em>Defer</em>') +
+                '</label>'
                 ))
         # Add a link back to the overview, if we're not viewing the overview!
         adminurl = mlist.GetScriptURL('admin', absolute=1)
@@ -284,15 +320,18 @@ def main():
             form.AddItem('<hr>')
             if not (details or sender or msgid or nomessages):
                 form.AddItem(Center(
+                    '<label>' +
                     CheckBox('discardalldefersp', 0).Format() +
                     '&nbsp;' +
-                    _('Discard all messages marked <em>Defer</em>')
+                    _('Discard all messages marked <em>Defer</em>') +
+                    '</label>'
                     ))
             form.AddItem(Center(SubmitButton('submit', _('Submit All Data'))))
         # Put 'Logout' link before the footer
+        doc.AddItem('\n<div align="right"><font size="+2">')
         doc.AddItem(Link('%s/logout' % admindburl,
-            '<div align="right"><font size="+2"><b>%s</b></font></div>' %
-            _('Logout')))
+            '<b>%s</b>' % _('Logout')))
+        doc.AddItem('</font></div>\n')
         doc.AddItem(mlist.GetMailmanFooter())
         print doc.Format()
         # Commit all changes
@@ -328,7 +367,7 @@ def show_pending_subs(mlist, form):
     form.AddItem('<hr>')
     form.AddItem(Center(Header(2, _('Subscription Requests'))))
     table = Table(border=2)
-    table.AddRow([Center(Bold(_('Address/name'))),
+    table.AddRow([Center(Bold(_('Address/name/time'))),
                   Center(Bold(_('Your decision'))),
                   Center(Bold(_('Reason for refusal')))
                   ])
@@ -341,12 +380,14 @@ def show_pending_subs(mlist, form):
     addrs.sort()
     num = 0
     for addr, ids in addrs:
-        # Eliminate duplicates
-        for id in ids[1:]:
+        # Eliminate duplicates.
+        # The list ws returned sorted ascending.  Keep the last.
+        for id in ids[:-1]:
             mlist.HandleRequest(id, mm_cfg.DISCARD)
-        id = ids[0]
-        time, addr, fullname, passwd, digest, lang = mlist.GetRecord(id)
+        id = ids[-1]
+        stime, addr, fullname, passwd, digest, lang = mlist.GetRecord(id)
         fullname = Utils.uncanonstr(fullname, mlist.preferred_language)
+        displaytime = time.ctime(stime)
         radio = RadioButtonArray(id, (_('Defer'),
                                       _('Approve'),
                                       _('Reject'),
@@ -357,11 +398,15 @@ def show_pending_subs(mlist, form):
                                          mm_cfg.DISCARD),
                                  checked=0).Format()
         if addr not in mlist.ban_list:
-            radio += '<br>' + CheckBox('ban-%d' % id, 1).Format() + \
-                     '&nbsp;' + _('Permanently ban from this list')
+            radio += ('<br>' + '<label>' +
+                     CheckBox('ban-%d' % id, 1).Format() +
+                     '&nbsp;' + _('Permanently ban from this list') +
+                     '</label>')
         # While the address may be a unicode, it must be ascii
         paddr = addr.encode('us-ascii', 'replace')
-        table.AddRow(['%s<br><em>%s</em>' % (paddr, Utils.websafe(fullname)),
+        table.AddRow(['%s<br><em>%s</em><br>%s' % (paddr,
+                                                   Utils.websafe(fullname),
+                                                   displaytime),
                       radio,
                       TextBox('comment-%d' % id, size=40)
                       ])
@@ -393,6 +438,7 @@ def show_pending_unsubs(mlist, form):
     num = 0
     for addr, ids in addrs:
         # Eliminate duplicates
+        # Here the order doesn't matter as the data is just the address.
         for id in ids[1:]:
             mlist.HandleRequest(id, mm_cfg.DISCARD)
         id = ids[0]
@@ -466,15 +512,19 @@ def show_helds_overview(mlist, form, ssort=SSENDER):
         left.AddRow([btns])
         left.AddCellInfo(left.GetCurrentRowIndex(), 0, colspan=2)
         left.AddRow([
+            '<label>' +
             CheckBox('senderpreserve-' + qsender, 1).Format() +
             '&nbsp;' +
-            _('Preserve messages for the site administrator')
+            _('Preserve messages for the site administrator') +
+            '</label>'
             ])
         left.AddCellInfo(left.GetCurrentRowIndex(), 0, colspan=2)
         left.AddRow([
+            '<label>' +
             CheckBox('senderforward-' + qsender, 1).Format() +
             '&nbsp;' +
-            _('Forward messages (individually) to:')
+            _('Forward messages (individually) to:') +
+            '</label>'
             ])
         left.AddCellInfo(left.GetCurrentRowIndex(), 0, colspan=2)
         left.AddRow([
@@ -490,9 +540,11 @@ def show_helds_overview(mlist, form, ssort=SSENDER):
         if mlist.isMember(sender):
             if mlist.getMemberOption(sender, mm_cfg.Moderate):
                 left.AddRow([
+                    '<label>' +
                     CheckBox('senderclearmodp-' + qsender, 1).Format() +
                     '&nbsp;' +
-                    _("Clear this member's <em>moderate</em> flag")
+                    _("Clear this member's <em>moderate</em> flag") +
+                    '</label>'
                     ])
             else:
                 left.AddRow(
@@ -503,9 +555,11 @@ def show_helds_overview(mlist, form, ssort=SSENDER):
                             mlist.reject_these_nonmembers +
                             mlist.discard_these_nonmembers):
             left.AddRow([
+                '<label>' +
                 CheckBox('senderfilterp-' + qsender, 1).Format() +
                 '&nbsp;' +
-                _('Add <b>%(esender)s</b> to one of these sender filters:')
+                _('Add <b>%(esender)s</b> to one of these sender filters:') +
+                '</label>'
                 ])
             left.AddCellInfo(left.GetCurrentRowIndex(), 0, colspan=2)
             btns = hacky_radio_buttons(
@@ -517,10 +571,11 @@ def show_helds_overview(mlist, form, ssort=SSENDER):
             left.AddCellInfo(left.GetCurrentRowIndex(), 0, colspan=2)
             if sender not in mlist.ban_list:
                 left.AddRow([
+                    '<label>' +
                     CheckBox('senderbanp-' + qsender, 1).Format() +
                     '&nbsp;' +
                     _("""Ban <b>%(esender)s</b> from ever subscribing to this
-                    mailing list""")])
+                    mailing list""") + '</label>'])
                 left.AddCellInfo(left.GetCurrentRowIndex(), 0, colspan=2)
         right = Table(border=0)
         right.AddRow([
@@ -656,13 +711,11 @@ def show_post_requests(mlist, id, info, total, count, form):
     for line in email.Iterators.body_line_iterator(msg, decode=True):
         lines.append(line)
         chars += len(line)
-        if chars > limit > 0:
+        if chars >= limit > 0:
             break
-    # Negative values mean display the entire message, regardless of size
-    if limit > 0:
-        body = EMPTYSTRING.join(lines)[:mm_cfg.ADMINDB_PAGE_TEXT_LIMIT]
-    else:
-        body = EMPTYSTRING.join(lines)
+    # We may have gone over the limit on the last line, but keep the full line
+    # anyway to avoid losing part of a multibyte character.
+    body = EMPTYSTRING.join(lines)
     # Get message charset and try encode in list charset
     # We get it from the first text part.
     # We need to replace invalid characters here or we can throw an uncaught
@@ -677,7 +730,7 @@ def show_post_requests(mlist, id, info, total, count, form):
     lcset = Utils.GetCharSet(mlist.preferred_language)
     if mcset <> lcset:
         try:
-            body = unicode(body, mcset).encode(lcset, 'replace')
+            body = unicode(body, mcset, 'replace').encode(lcset, 'replace')
         except (LookupError, UnicodeError, ValueError):
             pass
     hdrtxt = NL.join(['%s: %s' % (k, v) for k, v in msg.items()])
@@ -695,27 +748,25 @@ def show_post_requests(mlist, id, info, total, count, form):
     when = msgdata.get('received_time')
     if when:
         t.AddRow([Bold(_('Received:')), time.ctime(when)])
-        t.AddCellInfo(row+2, col-1, align='right')
-    # We can't use a RadioButtonArray here because horizontal placement can be
-    # confusing to the user and vertical placement takes up too much
-    # real-estate.  This is a hack!
-    buttons = Table(cellspacing="5", cellpadding="0")
-    buttons.AddRow(map(lambda x, s='&nbsp;'*5: s+x+s,
-                       (_('Defer'), _('Approve'), _('Reject'), _('Discard'))))
-    buttons.AddRow([Center(RadioButton(id, mm_cfg.DEFER, 1)),
-                    Center(RadioButton(id, mm_cfg.APPROVE, 0)),
-                    Center(RadioButton(id, mm_cfg.REJECT, 0)),
-                    Center(RadioButton(id, mm_cfg.DISCARD, 0)),
-                    ])
+        t.AddCellInfo(row+3, col-1, align='right')
+    buttons = hacky_radio_buttons(id,
+                (_('Defer'), _('Approve'), _('Reject'), _('Discard')),
+                (mm_cfg.DEFER, mm_cfg.APPROVE, mm_cfg.REJECT, mm_cfg.DISCARD),
+                (1, 0, 0, 0),
+                spacing=5)
     t.AddRow([Bold(_('Action:')), buttons])
-    t.AddCellInfo(row+3, col-1, align='right')
+    t.AddCellInfo(t.GetCurrentRowIndex(), col-1, align='right')
     t.AddRow(['&nbsp;',
+              '<label>' +
               CheckBox('preserve-%d' % id, 'on', 0).Format() +
-              '&nbsp;' + _('Preserve message for site administrator')
+              '&nbsp;' + _('Preserve message for site administrator') +
+              '</label>'
               ])
     t.AddRow(['&nbsp;',
+              '<label>' +
               CheckBox('forward-%d' % id, 'on', 0).Format() +
               '&nbsp;' + _('Additionally, forward this message to: ') +
+              '</label>' +
               TextBox('forward-addr-%d' % id, size=47,
                       value=mlist.GetOwnerEmail()).Format()
               ])
@@ -754,18 +805,18 @@ def process_form(mlist, doc, cgidata):
                 action = k[:len(prefix)-1]
                 qsender = k[len(prefix):]
                 sender = unquote_plus(qsender)
-                value = cgidata.getvalue(k)
+                value = cgidata.getfirst(k)
                 senderactions.setdefault(sender, {})[action] = value
                 for id in cgidata.getlist(qsender):
                     senderactions[sender].setdefault('message_ids',
                                                      []).append(int(id))
     # discard-all-defers
     try:
-        discardalldefersp = cgidata.getvalue('discardalldefersp', 0)
+        discardalldefersp = cgidata.getfirst('discardalldefersp', 0)
     except ValueError:
         discardalldefersp = 0
     # Get the summary sequence
-    ssort = int(cgidata.getvalue('summary_sort', SSENDER))
+    ssort = int(cgidata.getfirst('summary_sort', SSENDER))
     for sender in senderactions.keys():
         actions = senderactions[sender]
         # Handle what to do about all this sender's held messages
@@ -884,7 +935,7 @@ def process_form(mlist, doc, cgidata):
             forwardaddr = cgidata[forwardaddrkey].value
         # Should we ban this address?  Do this check before handling the
         # request id because that will evict the record.
-        if cgidata.getvalue(bankey):
+        if cgidata.getfirst(bankey):
             sender = mlist.GetRecord(request_id)[1]
             if sender not in mlist.ban_list:
                 # We don't need to validate the sender.  An invalid address
