@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2010 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2017 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@ from Mailman.htmlformat import *
 
 from Mailman.i18n import _
 
+from Mailman.CSRFcheck import csrf_token
+
 
 EMPTYSTRING = ''
 BR = '<br>'
@@ -38,8 +40,7 @@ COMMASPACE = ', '
 
 class HTMLFormatter:
     def GetMailmanFooter(self):
-        ownertext = COMMASPACE.join([Utils.ObscureEmail(a, 1)
-                                     for a in self.owner])
+        ownertext = Utils.ObscureEmail(self.GetOwnerEmail(), 1)
         # Remove the .Format() when htmlformat conversion is done.
         realname = self.real_name
         hostname = self.host_name
@@ -86,10 +87,14 @@ class HTMLFormatter:
         for person in people:
             id = Utils.ObscureEmail(person)
             url = self.GetOptionsURL(person, obscure=obscure)
+            person = self.getMemberCPAddress(person)
             if obscure:
                 showing = Utils.ObscureEmail(person, for_text=1)
             else:
                 showing = person
+            realname = Utils.uncanonstr(self.getMemberName(person), lang)
+            if realname and mm_cfg.ROSTER_DISPLAY_REALNAME:
+                showing += " (%s)" % Utils.websafe(realname)
             got = Link(url, showing)
             if self.getDeliveryStatus(person) <> MemberAdaptor.ENABLED:
                 got = Italic('(', got, ')')
@@ -314,12 +319,17 @@ class HTMLFormatter:
             container.AddItem("</center>")
         return container
 
-    def FormatFormStart(self, name, extra=''):
+    def FormatFormStart(self, name, extra='',
+                        mlist=None, contexts=None, user=None):
         base_url = self.GetScriptURL(name)
         if extra:
             full_url = "%s/%s" % (base_url, extra)
         else:
             full_url = base_url
+        if mlist:
+            return ("""<form method="POST" action="%s">
+<input type="hidden" name="csrf_token" value="%s">""" 
+                % (full_url, csrf_token(mlist, contexts, user)))
         return ('<FORM Method=POST ACTION="%s">' % full_url)
 
     def FormatArchiveAnchor(self):
